@@ -3,8 +3,8 @@
 # The class contains dimension description
 
 #' @importFrom  jsonlite toJSON
-DimensionModel <- function (data){
-  dimension.model=list(
+DimensionModel <- function (data) {
+  dimension.model = list(
     key = data$key,
     id = data$id,
     name = data$name
@@ -15,59 +15,54 @@ DimensionModel <- function (data){
   return(dimension.model)
 }
 
-DimensionModelList <- function (data){
-  dimension.model.list <- lapply(1:length(data),function(x) DimensionModel(data[[x]]))
+DimensionModelList <- function (data) {
+  dimension.model.list <- lapply(1:length(data), function(x) DimensionModel(data[[x]]))
   class(dimension.model.list) <- "DimensionModelList"
   return(dimension.model.list)
 }
 
 # The class contains dimension member information
-DimensionMember <- function(data){
-  dimension.member=list(
+DimensionMember <- function(data) {
+  dimension.member = list(
     key = data$key,
     name = data$name,
     level = data$level,
     hasdata = data$hasData,
     fields = data$fields
   )
-  dimension.member <- list2env(dimension.member)
-  class(dimension.member) <- "DimensionMember"
   return(dimension.member)
 }
 
 # The class contains dimension description and dimension items
-Dimension <- function (data){
-  dimension=list(
+Dimension <- function (data) {
+  dimension = list(
     dim.model = DimensionModel(data),
-    items=list(),
-    fields = list()
+    items = list(),
+    fields = list(),
+    key.map = list(),
+    id.map = list(),
+    name.map = list()
   )
-  dimension$items <- lapply(1:length(data$items),function(x) DimensionMember(data$items[x][[1]]))
+  dimension$items <- lapply(1:length(data$items), function(x) DimensionMember(data$items[x][[1]]))
   dimension$fields <- data$fields
 
+  dimension$key.map <- sapply(dimension$items, function(x) split(x, x$key))
+  dimension$name.map <- sapply(dimension$items, function(x) split(x, toupper(x$name)))
+  dimension$id.map <- sapply(dimension$items, function(x) if ('id' %in% names(x$fields)) split(x, toupper(x$fields$id)))
+
   # The method searches member of dimension by given member key
-  dimension$FindMemberByKey <- function(member.key){
-    for (item in dimension$items)
-      if (item$key == member.key)
-        return (item)
-    return (NULL)
+  dimension$FindMemberByKey <- function(member.key) {
+    return (dimension$key.map[[as.character(as.integer(member.key))]])
   }
 
   # The method searches member of dimension by given member id
-  dimension$FindMemberById <- function(member.id){
-    for (item in dimension$items) {
-      if ("id" %in% names(item$fields) & IsEqualStringsIgnoreCase(item$fields$id, member.id))
-        return (item)
-    }
-    return (NULL)
+  dimension$FindMemberById <- function(member.id) {
+    return (dimension$id.map[[toupper(as.character(member.id))]])
   }
 
   # The method searches member of dimension by given member name
-  dimension$FindMemberByName <-function (member.name){
-    for (item in dimension$items)
-      if (IsEqualStringsIgnoreCase(item$name, member.name))
-        return (item)
-    return (NULL)
+  dimension$FindMemberByName <- function (member.name) {
+    return (dimension$name.map[[toupper(member.name)]])
   }
   dimension <- list2env(dimension)
   class(dimension) <- "Dimension"
@@ -75,7 +70,7 @@ Dimension <- function (data){
 }
 
 # The class contains information about timeseries attributes
-TimeSeriesAttribute <- function (data){
+TimeSeriesAttribute <- function (data) {
   return (list(
     name = data$name,
     type = data$type,
@@ -85,20 +80,21 @@ TimeSeriesAttribute <- function (data){
 
 
 # The class contains dataset description
-Dataset <-function (data){
+Dataset <-function (data) {
   dataset = list(
     id = data$id,
+    type = data$type,
     dimensions = DimensionModelList(data$dimensions),
     time.series.attributes = list()
    )
 
-if (!is.null(data$timeseriesAttributes) && length(data$timeseriesAttributes)!=0){
-    dataset$time.series.attributes = lapply(1:length(data$timeseriesAttributes),function(x) TimeSeriesAttribute(data$timeseriesAttributes[x][[1]]))
+if (!is.null(data$timeseriesAttributes) && length(data$timeseriesAttributes) != 0) {
+    dataset$time.series.attributes <- lapply(1:length(data$timeseriesAttributes), function(x) TimeSeriesAttribute(data$timeseriesAttributes[x][[1]]))
   }
   # The method searching dimension with a given name
-  dataset$FindDimensionByName <- function (dim.name){
+  dataset$FindDimensionByName <- function (dim.name) {
     for (dim in dataset$dimensions) {
-      if (IsEqualStringsIgnoreCase(dim$name, dim.name)){
+      if (IsEqualStringsIgnoreCase(dim$name, dim.name)) {
         return (dim)
       }
     }
@@ -106,9 +102,9 @@ if (!is.null(data$timeseriesAttributes) && length(data$timeseriesAttributes)!=0)
   }
 
   # The method searching dimension with a given id
-  dataset$FindDimensionById <- function (dim.id){
-    for (dim in dataset$dimensions){
-      if (IsEqualStringsIgnoreCase(dim$id, dim.id)){
+  dataset$FindDimensionById <- function (dim.id) {
+    for (dim in dataset$dimensions) {
+      if (IsEqualStringsIgnoreCase(dim$id, dim.id)) {
         return (dim)
       }
     }
@@ -120,7 +116,7 @@ if (!is.null(data$timeseriesAttributes) && length(data$timeseriesAttributes)!=0)
 }
 
 # The class contains pivot request item
-PivotItem <- function(dimension.id,members){
+PivotItem <- function(dimension.id, members) {
   pivot.item = list(
     dimension.id = dimension.id,
     members = members
@@ -131,7 +127,7 @@ PivotItem <- function(dimension.id,members){
 }
 
 # The class contains pivot request item
-PivotTimeItem <- function(dimension.id=NULL, members=NULL, uimode=NULL){
+PivotTimeItem <- function(dimension.id=NULL, members=NULL, uimode=NULL) {
   pivot.time.item = list(
     dimension.id = dimension.id,
     members = members,
@@ -143,7 +139,7 @@ PivotTimeItem <- function(dimension.id=NULL, members=NULL, uimode=NULL){
 }
 
 # The class contains pivot request
-PivotRequest <- function(dataset){
+PivotRequest <- function(dataset) {
   pivot.request = list(
     dataset = dataset,
     header = list(),
@@ -154,24 +150,25 @@ PivotRequest <- function(dataset){
     set = function(x, value) pivot.request[[x]] <<- value
   )
   # Methods
-  pivot.request$GetItemArray <- function (items){
+  pivot.request$GetItemArray <- function (items) {
     arr <- list()
     i <- 1
-    if (length(items)>0){
-      for (item in items){
+    if (length(items)>0) {
+      for (item in items) {
         item.values <- list(
           "DimensionId" = item$dimension.id,
           "Members" = item$members
         )
-        if (is(item, "PivotTimeItem"))
+        if (is(item, "PivotTimeItem")) {
           item.values["UiMode"] <- item$uimode
+        }
         arr[[i]] <- item.values
         i <- i + 1
       }
     }
     return (arr)
   }
-  pivot.request$SaveToJson <- function(){
+  pivot.request$SaveToJson <- function() {
     request.values <- list(
       "Dataset"= pivot.request$get("dataset"),
       "Header" = pivot.request$GetItemArray(pivot.request$get("header")),
@@ -185,3 +182,6 @@ PivotRequest <- function(dataset){
   class(pivot.request) <- "PivotRequest"
   return(pivot.request)
 }
+
+
+
